// ParameterTable.cpp: implementation of the ParameterTable class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "glInteractive.h"
#include "ParameterTable.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParameterTable::ParameterTable()
{
	int i;

	for(i=0; i<NUM_BINS; i++)
		myBin[i]= NULL;
	myCount= 0;
}

ParameterTable::~ParameterTable()
{
	int i;
	Parameter *temp;

	for(i=0; i<NUM_BINS; i++) {
		while(myBin[i] != NULL) {
			temp= myBin[i];
			myBin[i]= myBin[i]->next;
			if(temp != NULL)
				delete temp;
		}

	}
}

int ParameterTable::Hash(const char *key)
{
	int i,val;

	val=0;
	for(i= strlen(key)-1; i>=0; i--)
		val+= (int)key[i];

	return val%NUM_BINS;
}

Parameter *ParameterTable::Add(const Parameter &newItem)
{
	int i;
	Parameter *temp;

	i= Hash(newItem.name);
	temp= new Parameter(newItem.name,newItem.min,newItem.max,newItem.val);
	if(temp != NULL) {
		temp->next= myBin[i];
		myBin[i]= temp;
		myCount++;
	}

	return temp;
}

Parameter *ParameterTable::Lookup(const char *key)
{
	int i;
	Parameter *temp;

	i= Hash(key);
	temp= myBin[i];
	while ((temp != NULL) && (strcmp(temp->name,key) != 0))
		temp= temp->next;

	return temp;
}

double ParameterTable::LookupValue(const char *key)
{
	Parameter *temp;

	temp= Lookup(key);
	if(temp != NULL)
		return temp->val;
	else
		return 0.0;
}

int ParameterTable::Gather(Parameter **items)
{
	int count,bin;
	Parameter *temp;

	count=0;
	for(bin=0; bin<NUM_BINS; bin++) {
		temp= myBin[bin];
		while (temp != NULL) {
			items[count++]= temp;
			temp= temp->next;
		}
	}

	return count;
}