// SimpleCloth.h: interface for the SimpleCloth class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SIMPLECLOTH_H__812DF867_B758_4CA3_8C27_590CE78C591B__INCLUDED_)
#define AFX_SIMPLECLOTH_H__812DF867_B758_4CA3_8C27_590CE78C591B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Vector.h"
#include "Matrix.h"
#include "ParameterTable.h"
#include <stdio.h>

class SimpleCloth
{
public:
	SimpleCloth();
	virtual int Create(double space,int nx,int ny,Matrix *CSystem=NULL);
	virtual ~SimpleCloth();
	bool Ready() {return curr != NULL;};
	double ShearK(double k=-1.0);
	double StretchK(double k=-1.0);
	double BendK(double k=-1.0);
	double DampingC(double c=-1.0);
	double Density(double rho=-1.0);
	virtual void glDraw();
	virtual void Increment(double dt=0.01);
	virtual Vector Position(double u,double v,Vector **source=NULL);
	virtual int MayaStartRecording(const char *fullpath,int num_u,int num_v);
	virtual int MayaRecordFrame(int frame=-1);
	virtual int MayaEndRecording();
	ParameterTable myParameters;
protected:
	virtual int WriteFrame(FILE *outfile,const char *name,int num_u,int num_v);
	Matrix coordSystem;
	Vector Normal(Vector **pvector,int ix,int iy);
	virtual void FreeMem();
	double windPhase,windPressure,windFrequency,windMinimum;
	Vector windDirection,windForce,windCenter;
	Vector gravity;
	Vector **curr,**prev,**next,**orig;
	double density;
	double spacing;
	double kStretch,kShear,kBend,cDamping;
	int nX,nY;

	struct MayaParameters{
		char name[20];
		int num_u,num_v,frame_no;
		FILE *outfile;
	} mayaParam;
};

#endif // !defined(AFX_SIMPLECLOTH_H__812DF867_B758_4CA3_8C27_590CE78C591B__INCLUDED_)
