// glInteractiveDoc.cpp : implementation of the CGlInteractiveDoc class
//

#include "stdafx.h"
#include "glInteractive.h"
#include "ClothInitializeDialog.h"
#include "RecSettingsDialog.h"
#include "glInteractiveDoc.h"
#include "glInteractiveView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveDoc

IMPLEMENT_DYNCREATE(CGlInteractiveDoc, CDocument)

BEGIN_MESSAGE_MAP(CGlInteractiveDoc, CDocument)
	//{{AFX_MSG_MAP(CGlInteractiveDoc)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_VIEW_CONTROLS, OnViewControls)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CONTROLS, OnUpdateViewControls)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveDoc construction/destruction

CGlInteractiveDoc::CGlInteractiveDoc()
{
	Matrix CS(4,4);
	double spacing=0.25;
	int i;
	int nx= 17;
	int ny= 10;

	
	for(i=0; i<16; i+=5)
		CS.GetElement()[i]= 1.0;
	CS.GetElement()[5]= -1.0;
	CS.GetElement()[10]= -1.0;
	CS.GetElement()[3]= -0.5*spacing*nx;
	CS.GetElement()[7]= 0.5*spacing*ny;
	mCloth.Create(spacing,nx,ny,&CS);
	m_pControls= new CClothControls(&mCloth.myParameters);
	m_pControls->Create(IDD_CONTROLS_DIALOG);
	m_showControls= false;
	m_recording= false;
	m_icount= 0;
	m_fcount= 0;
	m_iskip= 5;
	m_deltaT= 0.0005;
}

CGlInteractiveDoc::~CGlInteractiveDoc()
{
	if(m_pControls != NULL)
		delete m_pControls;
}

BOOL CGlInteractiveDoc::OnNewDocument()
{
	Matrix CS(4,4);
	int i;

	for(i=0; i<16; i+=5)
		CS.GetElement()[i]= 1.0;
	CS.GetElement()[5]= -1.0;
	CS.GetElement()[10]= -1.0;
	if (!CDocument::OnNewDocument())
		return FALSE;

	CClothInitializeDialog myDialog;
	if(myDialog.DoModal()== IDOK) {
		CS.GetElement()[3]= -0.5*myDialog.m_spacing*myDialog.m_nx;
		CS.GetElement()[7]= 0.5*myDialog.m_spacing*myDialog.m_ny;
		mCloth.Create(myDialog.m_spacing,myDialog.m_nx,myDialog.m_ny,&CS);
	}
	else {
		CS.GetElement()[3]= -0.5*0.25*17;
		CS.GetElement()[7]= 0.5*0.25*10;
		mCloth.Create(0.25,17,10,&CS);
	}

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveDoc serialization

void CGlInteractiveDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveDoc diagnostics

#ifdef _DEBUG
void CGlInteractiveDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CGlInteractiveDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveDoc commands

void CGlInteractiveDoc::OnFileSave() 
{
	char message[50];
	POSITION vPos;
	CRecSettingsDialog mySettings;
	CView *myView;

	sprintf(message,"Current delta t: %f1.3",m_deltaT);
	mySettings.m_static_message.SetWindowText(message);
	mySettings.m_skip= m_iskip= (int)(0.0333/m_deltaT);
	vPos= GetFirstViewPosition();
	myView= GetNextView(vPos);
	((CGlInteractiveView*)myView)->Pause();
	if(!m_recording){
		CFileDialog myDialog(false);
		if(myDialog.DoModal() == IDOK) {
			mySettings.DoModal();
			m_recording= (0==mCloth.MayaStartRecording(myDialog.GetPathName(),mySettings.m_num_u,mySettings.m_num_v));
			m_iskip= mySettings.m_skip;
			if(!m_recording)
				AfxMessageBox("Unable to start recording.",IDOK);
		}
	}
	else {
		m_recording= false;
		CWnd *wnd=AfxGetMainWnd();
		if(wnd)
			wnd->SetWindowText("Stopped recording");
		mCloth.MayaEndRecording();
	}
	((CGlInteractiveView*)myView)->Resume();
}

void CGlInteractiveDoc::OnViewControls() 
{
	m_showControls= !m_showControls;
	if(m_showControls)
		m_pControls->ShowWindow(SW_SHOW);
	else
		m_pControls->ShowWindow(SW_HIDE);
}

void CGlInteractiveDoc::OnUpdateViewControls(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_showControls == true ? 1 : 0);
}

void CGlInteractiveDoc::Increment()
{
	m_icount++;
	mCloth.Increment(m_deltaT);
	if(m_recording && (m_icount % m_iskip == 0)) {
		m_fcount++;
		CWnd *wnd=AfxGetMainWnd();
		if(wnd) {
			char message[50];
			sprintf(message,"Recording frame: %d",m_fcount);
			wnd->SetWindowText(message);
		}
		mCloth.MayaRecordFrame(m_fcount);
	}
}
