// glInteractiveView.cpp : implementation of the CGlInteractiveView class
//

#include "stdafx.h"
#include "glInteractive.h"
#include "glInteractiveDoc.h"
#include "glInteractiveView.h"
#include "Defines.h"
#include <GL/glu.h>
#include "glUtilities.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView

IMPLEMENT_DYNCREATE(CGlInteractiveView, CView)

BEGIN_MESSAGE_MAP(CGlInteractiveView, CView)
	//{{AFX_MSG_MAP(CGlInteractiveView)
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_COMMAND(ID_RUN_BUTTON, OnRunButton)
	ON_UPDATE_COMMAND_UI(ID_RUN_BUTTON, OnUpdateRunButton)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView construction/destruction

CGlInteractiveView::CGlInteractiveView()
{
	int i;

	m_viewMode= VIEW_SHADED;
	m_mouseMode= MOUSE_EXAM;
	m_perspDist= 20.0;
	m_orthoScale= 0.01;
	m_pCPal= NULL;
	m_hrc= NULL;
	for(i=0; i<16; i++)
		m_dOrientation[i]= (i%5 == 0) ? 1.0 : 0.0;

	m_pSelection= NULL;
}

CGlInteractiveView::~CGlInteractiveView()
{
	wglDeleteContext(m_hrc);
	if(m_pCPal != NULL)
		delete m_pCPal;
}

BOOL CGlInteractiveView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView drawing

void CGlInteractiveView::OnDraw(CDC* pDC)
{
	if (m_paletteMode == TRUE)
	{
		CPalette* pOldPal = pDC->SelectPalette(m_pCPal, TRUE);
		UINT u = pDC->RealizePalette();
	}
	GetRenderContext();
	glDraw();
	SwapBuffers(pDC->m_hDC);
	ReleaseRenderContext();
}

void CGlInteractiveView::glDraw()
{
    GLfloat materialColors[][4] = {{0.2f, 0.2f, 0.2f, 1.0f},
	                            {0.5f, 0.5f, 0.5f, 1.0f},
	                            {0.8f, 0.8f, 0.8f, 1.0f},
	                            {1.0f, 1.0f, 1.0f, 1.0f},
	                            {1.0f, 1.0f, 0.8f, 1.0f},
	                            {1.0f, 0.8f, 0.5f, 1.0f}};

    glClear(GL_COLOR_BUFFER_BIT	| GL_DEPTH_BUFFER_BIT);

	glEnable(GL_COLOR_MATERIAL);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, materialColors[2]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, materialColors[0]);

	GLfloat light_position[] = { -45.0f, -5.0f, 25.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	Matrix CS(4,4);
	CGlInteractiveDoc *pDoc;

	for(int i=0; i<16; i+=5)
		CS.GetElement()[i]= 1.0;
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glLoadIdentity();
	glMultMatrixd(m_dOrientation);		// multiply by orientation matrix
	if((pDoc= GetDocument())!= NULL) {
		glColor3d(0.75,0.75,0.75);
		pDoc->mCloth.glDraw();
	}
//	m_handle.glDraw(0);
	glDrawCoordinateSys(CS,2.0);

	glPopMatrix();
}

glObject *CGlInteractiveView::glPick(int x,int y)
{
	GLint viewport[4];
	glObject *result=NULL;
	RECT client;
    GLfloat materialColors[][4] = {{0.2f, 0.2f, 0.2f, 1.0f},
	                            {0.5f, 0.5f, 0.5f, 1.0f},
	                            {0.8f, 0.8f, 0.8f, 1.0f},
	                            {1.0f, 1.0f, 1.0f, 1.0f},
	                            {1.0f, 1.0f, 0.8f, 1.0f},
	                            {1.0f, 0.8f, 0.5f, 1.0f}};

	GetRenderContext();
    glClear(GL_COLOR_BUFFER_BIT	| GL_DEPTH_BUFFER_BIT);

	glEnable(GL_COLOR_MATERIAL);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, materialColors[2]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, materialColors[0]);

	GLfloat light_position[] = { 5.0f, 45.0f, 25.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	// get viewport metrics and set up pick transformation
	glGetIntegerv(GL_VIEWPORT,viewport);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	GetClientRect(&client);
	gluPickMatrix(x,client.bottom-y,2,2,viewport);
	glOrtho(-m_orthoScale*client.right,m_orthoScale*client.right,-m_orthoScale*client.bottom,m_orthoScale*client.bottom, -100.0, 100.0);
    
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glMultMatrixd(m_dOrientation);		// multiply by orientation matrix

	result= m_handle.glPick(x,y,0);		// selectable objects

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	
	ReleaseRenderContext();
	return result;
}

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView printing

BOOL CGlInteractiveView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGlInteractiveView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGlInteractiveView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView diagnostics

#ifdef _DEBUG
void CGlInteractiveView::AssertValid() const
{
	CView::AssertValid();
}

void CGlInteractiveView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGlInteractiveDoc* CGlInteractiveView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGlInteractiveDoc)));
	return (CGlInteractiveDoc*)m_pDocument;
}
#endif //_DEBUG


BOOL CGlInteractiveView::GetRenderContext()
{
// NEW CODE->    
	m_pDeviceContext= GetDC();
	ASSERT(m_pDeviceContext->m_hDC);
	if (m_hrc == NULL)
	{
		m_hrc = wglCreateContext(m_pDeviceContext->m_hDC);
		if(m_hrc == NULL)
		{
			TRACE("wglCreateContext Failed %x\r\n", GetLastError());
			return FALSE;
		}
	}
	return wglMakeCurrent(m_pDeviceContext->m_hDC, m_hrc);
// <-NEW CODE
/* OLD CODE->
	CDC *pdc = GetDC();
	ASSERT(pdc->m_hDC);
	if (m_hrc == NULL)
	{
		m_hrc = wglCreateContext(pdc->m_hDC);
		if(m_hrc == NULL)
		{
			TRACE("wglCreateContext Failed %x\r\n", GetLastError());
			return FALSE;
		}
	}

	return wglMakeCurrent(pdc->m_hDC, m_hrc);		// grab the rendering context
<-OLD CODE */
}

BOOL CGlInteractiveView::ReleaseRenderContext()
{
	ASSERT(m_pDeviceContext->m_hDC);
	ReleaseDC(m_pDeviceContext);
	return wglMakeCurrent(NULL, NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CGlInteractiveView message handlers

BOOL CGlInteractiveView::OnEraseBkgnd(CDC* pDC) 
{
	return true;
}

int CGlInteractiveView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CDC *pdc = GetDC();
	ASSERT(pdc->m_hDC);
    // set up pixel format
    glSetupPixelFormat(pdc->m_hDC);
	// set up color palette
	m_paletteMode= FALSE;
	TRACE("OnCreate : Palette\n");
	if (CreateRGBPalette(pdc->m_hDC, &m_pCPal))
	{
	    CPalette* pOldPal = pdc->SelectPalette(m_pCPal, FALSE);
	    UINT u = pdc->RealizePalette();
	    if (u != 0) InvalidateRect(NULL, TRUE);
		m_paletteMode = TRUE;
    }
	ReleaseDC(pdc);
	GetRenderContext();
	glInitialize();
	ReleaseRenderContext();

	m_timerID= 0;

	return 0;
}

void CGlInteractiveView::OnSize(UINT nType, int cx, int cy) 
{
	GetRenderContext();

	CView::OnSize(nType, cx, cy);
//	glReshapePerspective(cx,cy,m_perspDist);		// perspective
	glReshapeOrthographic(cx, cy, m_orthoScale);
	ReleaseRenderContext();
}

void CGlInteractiveView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_mouseDown= point;
	if((m_pSelection= glPick(point.x,point.y)) != NULL)
		m_mouseMode= MOUSE_DRAG_OBJECT;
	else
		m_mouseMode= MOUSE_EXAM;
	SetCapture();
}

void CGlInteractiveView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	if(m_mouseMode & MOUSE_EXAM)
		m_mouseMode|= MOUSE_ZOOM;
	else if(m_mouseMode & MOUSE_DRAG_OBJECT)
		m_pSelection= NULL;
	m_mouseDown= point;
	SetCapture();
}

void CGlInteractiveView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(GetCapture() == this)
		ReleaseCapture();
}

void CGlInteractiveView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	if(GetCapture() == this)
		ReleaseCapture();
}

void CGlInteractiveView::OnMouseMove(UINT nFlags, CPoint point) 
{
	double dx,dy,dz;
	double rotX[]={1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};
	double rotY[]={1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};
	RECT client;

	if(GetCapture() != this)
		return;

	if((nFlags & MK_LBUTTON) && (m_mouseMode & MOUSE_EXAM))
	{
		GetRenderContext();
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		
		dx= 0.005*(point.x-m_mouseDown.x);		// calculate offset angle about horizontal
		dy= 0.005*(point.y-m_mouseDown.y);		// calculate offset angle about vertical
		// calculate matrix for rotation about screen x axis
		rotX[5]= cos(dy);
		rotX[6]= sin(dy);
		rotX[9]= -sin(dy);
		rotX[10]= cos(dy);
		// calculate matrix for rotation about screen y axis
		rotY[0]= cos(dx);
		rotY[2]= -sin(dx);
		rotY[8]= sin(dx);
		rotY[10]= cos(dx);
		// calculate the incremental change in orientation due to the screen mouse movement
		glLoadMatrixd(rotY);
		glMultMatrixd(rotX);
		glMultMatrixd(m_dOrientation);
		// store the result in the orientation matrix
		glGetDoublev(GL_MODELVIEW_MATRIX,m_dOrientation);

		glPopMatrix();

		ReleaseRenderContext();

		m_mouseDown= point;
		InvalidateRect(NULL);
	}
	else if((nFlags == MK_RBUTTON) && (m_mouseMode & MOUSE_ZOOM))
	{
		GetClientRect(&client);
		dz= 1.0*(m_mouseDown.y - point.y)/client.bottom;
		dz= (dz > 1.0) ? dz=0.95 : dz;
		m_orthoScale*= (1.0-dz);

		m_mouseDown= point;

		GetRenderContext();
		glReshapeOrthographic(client.right,client.bottom,m_orthoScale);

		ReleaseRenderContext();
		InvalidateRect(NULL);
	}
	else if((nFlags == MK_LBUTTON) && (m_mouseMode & MOUSE_DRAG_OBJECT))
	{
		GetClientRect(&client);
		GetRenderContext();
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();

		glLoadIdentity();
		glMultMatrixd(m_dOrientation);		// multiply by orientation matrix
		m_pSelection->Drag(0,0,point.x,point.y,X_FREE | Y_FREE);
		glPopMatrix();
		ReleaseRenderContext();
		InvalidateRect(NULL);
	}
}

void CGlInteractiveView::OnTimer(UINT nIDEvent) 
{
	CView::OnTimer(nIDEvent);
	if(nIDEvent == m_timerID){
		IncrementAnimation();
		InvalidateRect(NULL);
	}
}

void CGlInteractiveView::IncrementAnimation()
{
	double rotY[]={1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};
	static bool updating=false;
	CGlInteractiveDoc *pDoc;

	if(updating)
		return;
	updating= true;
	if((pDoc= GetDocument())!= NULL)
		pDoc->Increment();
	updating= false;
}

void CGlInteractiveView::OnDestroy() 
{
	CView::OnDestroy();
	if(m_timerID != 0)
		KillTimer(m_timerID);
}

void CGlInteractiveView::OnRunButton() 
{
	if(m_timerID == 0)
		m_timerID= CView::SetTimer(ID_ANIMATION_TIMER,10,NULL);
	else {
		CView::KillTimer(m_timerID);
		m_timerID= 0;
	}
}

void CGlInteractiveView::OnUpdateRunButton(CCmdUI* pCmdUI) 
{
	if(m_timerID != 0)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
}

void CGlInteractiveView::Pause()
{
	if(m_timerID != 0) {
		CView::KillTimer(m_timerID);
		m_timerID= 0;
	}
}

void CGlInteractiveView::Resume()
{
	if(m_timerID == 0)
		m_timerID= CView::SetTimer(ID_ANIMATION_TIMER,10,NULL);
}
